#!perl

# vim: ts=4 sts=4 sw=4 et: syntax=perl
#
# Copyright (c) 2021-2023 Sven Kirmess
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

use 5.006;
use strict;
use warnings;

use Test::More 0.88;

use Cwd            ();
use File::Basename ();
use File::Spec     ();
use lib File::Spec->catdir( File::Basename::dirname( Cwd::abs_path __FILE__ ), 'lib' );

use Local::Test::Exception qw(exception);
use Local::Test::TempDir   qw(tempdir);

use Git::Background 0.003;

note(q{new( { invalid_argument => 17 } )});
{
    like( exception { Git::Background->new( { invalid_argument => 17 } ) }, qr{\QUnknown argument: 'invalid_argument'\E}, 'new throws an exception for an invalid argument' );
}

note(q{new( { invalid_argument => 17, another_arg => 1 } )});
{
    like( exception { Git::Background->new( { invalid_argument => 17, another_arg => 1 } ) }, qr{\QUnknown arguments: 'another_arg', 'invalid_argument'\E}, 'new throws an exception for invalid arguments' );
}

note(q{new( $dir, { dir => $dir } )});
{
    my $dir = tempdir();
    like( exception { Git::Background->new( $dir, { dir => $dir } ); }, qr{\A\QCannot specify dir as positional argument and in argument hash\E}, 'throws an exception if dir is specified twice' );
}

note(q{to many/wrong arguments});
{
    my $dir = tempdir();
    like( exception { Git::Background->new( $dir, { fatal => 0, git => 'my-git' }, 'hello world' ) }, qr{\Qusage: new( [DIR], [ARGS] )\E}, 'new throws an exception with to many arguments' );
    like( exception { Git::Background->new( $dir, 'hello world' ) }, qr{\Qusage: new( [DIR], [ARGS] )\E}, 'new throws an exception with wrong argument' );
    like( exception { Git::Background->new( {},   {} ) },            qr{\Qusage: new( [DIR], [ARGS] )\E}, 'new throws an exception with wrong argument' );
}

#
done_testing();

exit 0;
